@@ -1,18 +1,37 @@
 # pylint: disable=C0103, C0303
+from __future__ import absolute_import
 import numpy as np
 import pandas as pd
 from pandas import DataFrame
 from lxml import objectify
 from fitparse import FitFile
-import tcxtools
-from utils import totimestamp, geo_distance
+try:
+    from . import tcxtools
+    from .utils import totimestamp, geo_distance
+except (ValueError,ImportError):
+    import tcxtools
+    from utils import totimestamp, geo_distance
+
+import sys
+if sys.version_info[0]<=2:
+    pythonversion = 2
+else:
+    pythonversion = 3
+
 import gzip
 import arrow
 import shutil
 from datetime import datetime
+from six.moves import range
+import json
 
 NAMESPACE = 'http://www.garmin.com/xmlschemas/TrainingCenterDatabase/v2'
 
+def strip_non_ascii(string):
+    ''' Returns the string without non ASCII characters'''
+    stripped = (c for c in string if 0 < ord(c) < 127)
+    return ''.join(stripped)
+
 def tofloat(x):
     try:
         return float(x)
@@ -30,7 +49,7 @@ class ExcelTemplate(object):
         unixnow = arrow.get(now).timestamp
         time = 0
         totdistance = 0
-        
+
         for nr,row in self.xls_df.iterrows():
             duration = row['Interval Time']
             #duration = datetime.strptime(durationstring,"%M:%S.%f")
@@ -43,15 +62,15 @@ class ExcelTemplate(object):
                 if spm == np.nan:
                     spm = 10.
                 deltat = 60./spm
-                aantal = int(seconds/deltat)+1
+                aantal = max(2,int(seconds/deltat)+1)
                 time_list = time+np.arange(aantal)*deltat
                 distance = row['Interval Distance']
-                deltad = distance/float(aantal)
+                deltad = distance/float(aantal-1)
                 d_list = np.arange(aantal)*deltad
-                
+
                 velo = distance/float(seconds)
                 pace = 500./velo
-                
+
                 data = pd.DataFrame({
                     'time':time_list,
                     'distance':d_list,
@@ -64,7 +83,7 @@ class ExcelTemplate(object):
                 })
 
 
-                
+
                 self.df = self.df.append(data)
                 time += seconds
                 totdistance = distance
@@ -97,7 +116,7 @@ class ExcelTemplate(object):
                     else:
                         velo = 0
                         pace = 0
-                    
+
                     data = pd.DataFrame({
                         'time':time_list,
                         'distance':d_list,
@@ -128,7 +147,7 @@ class ExcelTemplate(object):
         writeFile = args[0]
 
         data = self.df
-        
+
         if isgzip:
             return data.to_csv(writeFile + '.gz', index_label='index',
                                compression='gzip')
@@ -150,7 +169,7 @@ class FitSummaryData(object):
         lapdict = []
         lapcounter = 0
         for record in self.records:
-            if record.name == 'record':
+            if record.name ==  'record':
                 values = record.get_values()
                 values['lapid'] = lapcounter
                 recorddicts.append(values)
@@ -162,7 +181,7 @@ class FitSummaryData(object):
 
         self.df = pd.DataFrame(recorddicts)
         self.lapdf = pd.DataFrame(lapdict)
-                
+
         self.summarytext = 'Work Details\n'
 
 
@@ -188,6 +207,7 @@ class FitSummaryData(object):
             inttime = self.lapdf[self.lapdf['lapid']==lapcount+1][
                 'total_elapsed_time'
             ]
+            inttime = float(inttime)
             try:
                 intpower = int(group['power'].mean())
             except KeyError:
@@ -195,11 +215,16 @@ class FitSummaryData(object):
             lapmin = int(inttime/60)
             lapsec = int(int(10*(inttime-lapmin*60.))/10.)
             try:
-                intvelo = group['speed'].mean()
+                intvelo = group['enhanced_speed'].mean()
                 intpace = 500./intvelo
             except KeyError:
-                intvelo = 0
-                intpace = 0
+                try:
+                    intvelo = group['speed'].mean()
+                    intpace = 500./intvelo
+                except KeyError:
+                    intvelo = 0
+                    intpace = 0
+
             pacemin = int(intpace/60)
             pacesec = int(10*(intpace-pacemin*60.))/10.
             pacestring = str(pacemin)+":"+str(pacesec)
@@ -211,7 +236,7 @@ class FitSummaryData(object):
                 intdps = intdist/float(strokecount)
             except ZeroDivisionError:
                 intdps = 0.0
-                
+
             summarystring = "{nr:0>2}{sep}{intdist:0>5d}{sep}".format(
                 nr=lapcount+1,
                 sep=separator,
@@ -238,7 +263,7 @@ class FitSummaryData(object):
                 intpower=intpower,
                 sep=separator
             )
-                
+
             summarystring += " {inthr:0>3d} {sep}".format(
                 inthr=inthr,
                 sep=separator
@@ -257,10 +282,14 @@ class FitSummaryData(object):
             self.summarytext += summarystring
 
         # add total summary
-        overallvelo = self.df['speed'].mean()
+        try:
+            overallvelo = self.df['enhanced_speed'].mean()
+        except KeyError:
+            overallvelo = self.df['speed'].mean()
+
         timestamps = self.df['timestamp'].apply(totimestamp)
         totaltime = timestamps.max()-timestamps.min()
-            
+
         overallpace = 500./overallvelo
 
         minutes = int(overallpace/60)
@@ -289,7 +318,7 @@ class FitSummaryData(object):
             avgdps = totaldistance/strokecount
         except ZeroDivisionError:
             avgdps = 0
-            
+
 
         summarystring = "Workout Summary\n"
         summarystring += "--{sep}{totaldistance:0>5}{sep}".format(
@@ -342,10 +371,10 @@ class FITParser(object):
         self.fitfile = FitFile(self.readfile, check_crc=False)
 
         self.records = self.fitfile.messages
-                
+
         recorddicts = []
         lapcounter = 0
-        
+
         for record in self.records:
             if record.name == 'record':
                 values = record.get_values()
@@ -354,18 +383,48 @@ class FITParser(object):
             if record.name == 'lap':
                 lapcounter += 1
 
-        
+
 
         self.df = pd.DataFrame(recorddicts)
 
-        latitude = self.df['position_lat']*(180./2**31)            
-        longitude = self.df['position_long']*(180./2**31)
+        # columns to lowercase - this should be easier
+        self.df.columns = [strip_non_ascii(x) for x in self.df.columns]
+        self.df.columns = [x.encode('ascii','ignore') for x in self.df.columns]
+        if pythonversion == 3:
+            #        self.df.columns = [str(x) for x in self.df.columns]
+            self.df.columns = [x.decode('ascii') for x in self.df.columns]
 
-        distance = self.df['distance']
+        self.df.rename(columns = str.lower,inplace=True)
+
+
+        # check column dimensions
+
+        for c in self.df.columns:
+            x = self.df[c]
+            if len(x.shape)>1:
+                newdf = pd.DataFrame({
+                    c: x.iloc[:,0].values
+                    })
+                self.df.drop(labels=c,axis=1,inplace=True)
+                self.df[c] = newdf[c]
+
+        try:
+            latitude = self.df['position_lat']*(180./2**31)
+            longitude = self.df['position_long']*(180./2**31)
+        except KeyError:
+            # no coordinates
+            latitude = 0
+            longitude = 0
+
+        try:
+            distance = self.df['distance']
+        except KeyError:
+            distance = pd.Series(np.zeros(len(self.df)))
 
         self.df['position_lat'] = latitude
         self.df['position_long'] = longitude
 
+
         if pd.isnull(distance).all():
             dist2 = np.zeros(len(distance))
             for i in range(len(distance)-1):
@@ -379,7 +438,18 @@ class FITParser(object):
                 dist2[i+1] = dist2[i]+deltal
             self.df['distance'] = dist2
 
-        velo = self.df['speed']
+        try:
+            velo = self.df['enhanced_speed']
+        except KeyError:
+            try:
+                velo = self.df['speed']
+            except KeyError:
+                velo = pd.Series(np.zeros(len(self.df)))
+
+        if velo.mean() >= 1000:
+            velo = velo/1000.
+
+
         timestamps = self.df['timestamp'].apply(totimestamp)
         pace = 500./velo
         elapsed_time = timestamps-timestamps.values[0]
@@ -387,7 +457,7 @@ class FITParser(object):
         self.df['TimeStamp (sec)'] = timestamps
         self.df[' Stroke500mPace (sec/500m)'] = pace
         self.df[' ElapsedTime (sec)'] = elapsed_time
-        
+
         newcolnames = {
             'power': ' Power (watts)',
             'heart_rate': ' HRCur (bpm)',
@@ -414,6 +484,35 @@ class FITParser(object):
         else:
             return self.df.to_csv(writefile, index_label='index')
 
+class JSONParser(object):
+    def __init__(self, json_file):
+        df = pd.DataFrame()
+        with open(json_file,'r') as f:
+            data = json.load(f)
+
+        laps = data['laps']
+
+        for lap in laps:
+            points = lap['points']
+            ldf = pd.DataFrame.from_records(points)
+            df=df.append(ldf,ignore_index=True)
+
+
+        self.df = df
+
+        newcolnames = {
+            'time':'TimeStamp (sec)',
+            'hr':' HRCur (bpm)',
+            }
+
+        self.df.rename(columns=newcolnames,inplace=True)
+
+
+    def write_csv(self,writefile="json_o.csv", gzip = False):
+        if gzip:
+            return self.df.to_csv(writefile+'.gz', index_label='index',compression='gzip')
+        else:
+            return self.df.to_csv(writefile, index_label='index')
 
 class TCXParserTester(object):
     def __init__(self, tcx_file):
@@ -456,8 +555,17 @@ class TCXParserTester(object):
 
 
 class TCXParser(object):
-    def __init__(self, tcx_file):
-        self.df = tcxtools.tcxtodf(tcx_file)
+    def __init__(self, tcx_file, *args, **kwargs):
+        if 'alternative' in kwargs:
+            alternative = kwargs['alternative']
+        else:
+            alternative = False
+
+        if alternative:
+            self.df = tcxtools.tcxtodf(tcx_file)
+        else:
+            self.df = tcxtools.tcxtodf2(tcx_file)
+
         try:
             lat = self.df['latitude'].apply(tofloat).values
             longitude = self.df['longitude'].apply(tofloat).values
@@ -466,7 +574,7 @@ class TCXParser(object):
             self.df['longitude'] = 0
             lat = self.df['latitude'].apply(tofloat).values
             longitude = self.df['longitude'].apply(tofloat).values
-            
+
 
         unixtimes = self.df['timestamp'].values
         try:
@@ -477,10 +585,10 @@ class TCXParser(object):
                 self.df['Cadence'] = self.df['StrokeRate']
             except KeyError:
                 try:
-                    spm = 0.0*self.df['Speed']
+                    spm = 0.0*self.df['Speed'].apply(tofloat).values
                 except KeyError:
                     spm = 0.0*unixtimes
-                    
+
 
         try:
             velo = self.df['Speed'].apply(tofloat)
@@ -499,15 +607,20 @@ class TCXParser(object):
                     velo[i+1] = deltal/(1.0*(unixtimes[i+1]-unixtimes[i]))
                 except ZeroDivisionError:
                     velo[i+1] = velo[i]
-                if spm[i] <> 0:
+                if spm[i] != 0:
                     strokelength[i] = deltal*60/spm[i]
                 else:
                     strokelength[i] = 0.
-        
+
         try:
             power = self.df['Watts']
         except KeyError:
-            self.df['Watts'] = 0*spm
+            try:
+                power = self.df['ns3:Watts']
+            except KeyError:
+                power = 0*spm
+
+            self.df['Watts'] = power
 
         p = 500./velo
 
@@ -536,7 +649,7 @@ class TCXParser(object):
                     self.df[c] = self.df[c].astype(float)
                 except KeyError:
                     pass
-                    
+
 
     def write_csv(self, writefile='example.csv', window_size=5, gzip=False):
         data = self.df
@@ -551,7 +664,7 @@ class TCXParser(object):
                 data = data.drop(c, axis=1)
             if c == 'Extensions':
                 data = data.drop(c, axis=1)
-                        
+
         if gzip:
             return data.to_csv(writefile+'.gz', index_label='index',
                                compression='gzip')
